<?php

class Garam_Admin_Form
{

    const ID = 'garam-livechat-chatbot';

    const NONCE_KEY = 'garam_admin';

    protected $views = array(
        'admin-view' => 'admin-view',
    );

    const WHITELISTED_KEYS = array(
        'garam-admin-token',
    );

    private $default_values = array();

    public function __construct() {
        add_action('wp_enqueue_scripts', array($this, 'add_site_domain_script'));
    }

    public function init()
    {
        add_action('admin_menu', array($this, 'add_menu_page'), 20);
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'));
        add_action('rest_api_init', array($this, 'add_endpoint'));
        add_action('admin_post_garam_admin_save', array($this, 'submit_save'));
        add_action('wp_enqueue_scripts', array($this, 'cp_front_custom_script'));
        add_action('wp_footer', array($this, 'cp_insert_my_footer'));
    }

    public function get_id()
    {
        return self::ID;
    }

    public function get_nonce_key()
    {
        return self::NONCE_KEY;
    }

    public function get_whitelisted_keys()
    {
        return self::WHITELISTED_KEYS;
    }

    private function get_defaults()
    {
        $defaults = array();
        foreach ($this->get_whitelisted_keys() as $key => $val) {
            $defaults[$val] = get_option($val);
        }
        return $defaults;
    }


    public function add_menu_page()
    {
        add_menu_page(
            'Garam - LiveChat & ChatBot',
            'Garam - LiveChat & ChatBot',
            'manage_options',
            $this->get_id(),
            array(&$this, 'load_view'),
            'dashicons-airplane'
        );
    }

    function load_view()
    {
        $this->default_values = $this->get_defaults();
        $group = "garam-admin-token";
        $args['garam_admin_token'] = isset($this->default_values[$group]) ? stripslashes($this->default_values[$group]) : '';

        /**
         * Default Admin Form Template
         */

        echo '<div class="garam-admin-forms admin-view">';
        echo '<div class="container container1 col-md-12 ">';
        echo '<div class="inner">';

        $this->includeWithVariables(garam_admin_template_server_path('views/alerts', false));
        $this->includeWithVariables(garam_admin_template_server_path('views/admin-view', false), $args);

        echo '</div>';
        echo '</div>';
        echo '</div> <!-- / garam-admin-forms -->';
    }

    function includeWithVariables($filePath, $variables = array(), $print = true)
    {
        $output = NULL;
        if (file_exists($filePath)) {
            // Extract the variables to a local namespace
            extract($variables);
            // Start output buffering
            ob_start();
            // Include the template file
            include $filePath;
            // End buffering and return its contents
            $output = ob_get_clean();
        }
        if ($print) {
            print $output;
        }
        return $output;
    }


    public function admin_enqueue_scripts($hook)
    {
        if ($hook !== 'toplevel_page_garam-livechat-chatbot') {
            return;
        }

        wp_enqueue_style('garam-admin-form-bs', GARAM_URL . 'assets/bootstrap/bootstrap.min.css', array(), GARAM_WP_ADMIN_VERSION, false);
        wp_enqueue_script('garam-admin-form-bs', GARAM_URL . 'assets/bootstrap/bootstrap.bundle.min.js', array(), GARAM_WP_ADMIN_VERSION, true);
        wp_enqueue_style('garam-admin-form', GARAM_URL . 'assets/style.css', array(), GARAM_WP_ADMIN_VERSION, false);
        wp_enqueue_style('garam-admin-form', GARAM_URL . 'assets/addon.js', array(), GARAM_WP_ADMIN_VERSION, true);
    }

    public function submit_save()
    {
        $nonce = sanitize_text_field(wp_unslash($_POST[$this->get_nonce_key()]));
        $action = sanitize_text_field(wp_unslash($_POST['action']));
        
        if (!isset($nonce) || !wp_verify_nonce($nonce, $action)) {
            print 'Sorry, your nonce did not verify.';
            exit;
        }
        
        if (!current_user_can('manage_options')) {
            print 'You can\'t manage options';
            exit;
        }

        /**
         * whitelist keys that can be updated
         */
        $whitelisted_keys = $this->get_whitelisted_keys();
        $fields_to_update = [];

        foreach ($whitelisted_keys as $key) {
            if (array_key_exists($key, $_POST)) {
                $fields_to_update[$key] = sanitize_text_field(wp_unslash($_POST[$key]));
            }
        }

        /**
         * Loop through form fields keys and update data in DB (wp_options)
         */

        $this->db_update_options($fields_to_update);
        $redirect_to = sanitize_text_field(wp_unslash($_POST['redirectToUrl']));

        if ($redirect_to) {
            add_settings_error('ct_msg', 'ct_msg_option', __("Changes saved.", "garam-livechat-chatbot"), 'success');
            set_transient('settings_errors', get_settings_errors(), 30);
            wp_safe_redirect($redirect_to);
            exit;
        }
    }

    private function db_update_options($group)
    {
        foreach ($group as $key => $fields) {
            $db_opts = get_option($key);
            $db_opts = ($db_opts === '') ? array() : $db_opts;
            update_option($key, $fields);
        }
    }


    public function add_endpoint()
    {
        register_rest_route('garam-livechat-chatbot', '(?P<path>.+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'relay_get_api_call'),
        ));
    }

    public function relay_get_api_call($request)
    {
        $endpoint = GARAM_API_URL . $request->get_param('path');

        $args = array(
            'headers'     => array(
                'Content-Type' => 'application/json',
                'token' => get_option('garam-admin-token')
            )
        );

        $response = wp_remote_get($endpoint, $args);

        if (is_wp_error($response)) {
            return new WP_Error( 'external_api_error', 'Error fetching external API.', array( 'status' => 500 ) );
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body);


        return new WP_REST_Response($data, 200, array('Access-Control-Allow-Origin' => '*'));
    }

    public function cp_insert_my_footer()
    {
        $this->add_site_domain_script();
        add_action('wp_enqueue_scripts', array($this, 'add_site_domain_script'));
    }

    public function add_site_domain_script()
    {
        $site_domain = esc_js(garam_current_site_domain());
        $domain_script = "window.garamService = {siteDomain: '{$site_domain}', siteType: 'WORDPRESS'};";

        wp_register_script('site-domain-script', '', [], GARAM_WP_ADMIN_VERSION, true);
        wp_add_inline_script( 'site-domain-script', $domain_script );
        wp_enqueue_script('site-domain-script');
    }

    public function cp_front_custom_script()
    {
        wp_enqueue_script('ai-everywhere-chatbot-app-script', GARAM_CHAT_BOT_ADD_ON_FILE_URL, array('site-domain-script'), GARAM_WP_ADMIN_VERSION, true);
    }
}


